;;;  Contains: Graphics code to represent UA & TO plans (not perfect)
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)

;(use-package :wt)
;(use-package :cw)

(defstruct pwin
  (window nil)
  (columns nil)
  (cells nil))

(defstruct (pcell (:print-function
		   (lambda (struct stream depth)
		     (print-pcell struct stream depth))))
  (x 0)
  (y 0)
  (pwin nil)
  (step nil))


(defvar *graphics-on* t)

(defvar *CELL-SIZE-X* 40)
(defvar *CELL-SIZE-Y* 40)
(defvar *spacing* 30)
(defvar *border* 20)

(defvar *col1* .13)
(defvar *col2* .39)
(defvar *col5* .32)
(defvar *col4* .5)
(defvar *col3* .74)
(defvar *row1* .15)
(defvar *row2* .30)
(defvar *row3* .45)
(defvar *row4* .65)

(defvar *WINDOW-WIDTH*  0)
(defvar *WINDOW-HEIGHT* 0)
(defvar *graphics-depth* 0)
(defvar *old-numbers* nil)
(defvar *SCREEN-ARRAY* nil)
(defvar *screen-pointer* 0)
(defvar *PLAN-SCREEN* nil)
(defvar *pause-graphics* 0.5)
(defvar *next-step* nil)
(defvar *active-left* nil)
(defvar *active-middle* nil)
(defvar *active-right* nil)
(defvar *streamline* nil)


(defun init-graphics (depth-level plan title &key ((:force force) nil))
 (if *graphics-on*
  (progn
    (setf *graphics-depth* depth-level)
    (let ((dim (from-cell-to-screen (+ 2 depth-level) depth-level))
	  (buffer (from-cell-to-screen 0 0))
	  (new-numbers nil))
      (setf *window-width*  (incf (car dim) (car buffer)))
      (setf *window-height* (incf (cdr dim) (cdr buffer)))
      (setf new-numbers `(,*window-width* ,*window-height* ,*graphics-depth*))
      (if (not (equal new-numbers *old-numbers*)) (setf force t))
      (setf *old-numbers* new-numbers))
    (cond (force
	   (if (typep *screen-array* 'array)
	       (dotimes (index (array-dimension *screen-array* 0))
		  (close (pwin-window (aref *Screen-Array* index)))))
	   (if *plan-screen* (close *plan-screen*))
	   (setf *plan-screen* nil
		 *screen-array* (make-array (1+ depth-level)))
	   (dotimes (i (1+ depth-level))
		    (let* ((win (make-window-stream
				 :left   10 :bottom 10 :width  *window-width*
				 :height *window-height* :activate-p nil))
			   (pwin (make-pwin :window win :columns
					    (make-array (+ 2 depth-level)))))
		      (setf (pwin-cells pwin)
			    (loop for j from 0 to (1+ i)
				  collect (make-pcell :pwin pwin)))
		      (setf (aref *screen-array* i) pwin)))
	   (setf *plan-screen* (make-window-stream
				:left 10 :bottom 10 :activate-p t :title title
				:height *window-height* :width *window-width*))
	   (setf *screen-pointer* 0)
	   (modify-window-stream-method
	    *plan-screen* :repaint :after 'redraw-plan)
	   (testing))
	  (t
	   (setf (window-stream-title *plan-screen*) title)
	   (setf *screen-pointer* 0)))
    (enscribe-plan plan *screen-pointer*)
    (repaint *plan-screen*)
    nil)))


(defun graphics-add-step (plan depth)
  (if *graphics-on*
      (progn
	(setf *screen-pointer* (1+ depth))
	(enscribe-plan (reverse plan) (1+ depth))
	(help-redraw-plan *plan-screen* (1+ depth))
	(if (and (numberp *pause-graphics*) (< 0 *pause-graphics*))
	    (sleep *pause-graphics*)))))


(defun graphics-del-step (plan depth)
  (declare (ignore plan))
  (if *graphics-on*
      (progn
	(if *streamline*
	    (progn
	      (mouse-cursor-left-handler nil nil)
	      (setf *next-step* nil)
	      (mp:process-wait "Planner" #'(lambda nil *next-step*))
	      (mouse-cursor-left-handler nil nil)))
	(help-redraw-plan *plan-screen* (setf *screen-pointer* depth)))))


(defun enscribe-plan (plan pwin-index)
  (let ((tuple nil)
	(pwin (aref *screen-array* pwin-index)))
    (clear (pwin-window pwin))
    (loop for index from 0 to (1+ *graphics-depth*)
	  do (setf (aref (pwin-columns pwin) index) (1- *graphics-depth*)))
    (loop with cells = (pwin-cells pwin)
	  for step in plan
	  do (setf (pcell-step (car cells)) step
		   tuple (assoc pwin (operator-graphics step)))
	     (if tuple (setf (cdr tuple) (car cells))
	       (push (cons pwin (car cells)) (operator-graphics step)))
	     (pop cells))
    (draw-the-cells pwin)
    (draw-the-arcs  pwin)))


(defun draw-the-cells (pw)
  (mark-for-depth *initial-operator* 0)
  (dolist (pc (pwin-cells pw))
    (setf (pcell-x pc) (operator-mark (pcell-step pc)))
    (setf (pcell-y pc) (1+ (decf (aref (pwin-columns pw) (pcell-x pc)))))
    (let ((name (operator-class-name (operator-class (pcell-step pc)))))
      (cond ((equal *initial-operator* (pcell-step pc))
	     (write-cell (pcell-x pc) (pcell-y pc) '+
			 nil nil nil (pwin-window pw)))
	    ((equal *final-operator* (pcell-step pc))
	     (write-cell (pcell-x pc) (pcell-y pc) '-
			 nil nil nil (pwin-window pw)))
	    (t
	     (write-cell (pcell-x pc) (pcell-y pc) (nth 1 name)
			 (nth 2 name) (nth 3 name)
			 (operator-instance (pcell-step pc))
			 (pwin-window pw)))))))
	

(defun draw-the-arcs (pwin)
  (dolist (pc1 (pwin-cells pwin))
    (let ((step1 (pcell-step pc1)) (pc2 nil))
      (dolist (step2 (operator-succs step1))
	(setf pc2 (cdr (assoc pwin (operator-graphics step2))))
	(if (draw-arc? step1 step2)
	    (graphics-draw-arc pc1 pc2 pwin))))))


(defun graphics-draw-arc (pc1 pc2 pwin)
  (let ((l1 (from-cell-to-screen (pcell-x pc1) (pcell-y pc1)))
	(l2 (from-cell-to-screen (pcell-x pc2) (pcell-y pc2))))
    (incf (cdr l1) (/ *CELL-SIZE-Y* 2))
    (incf (car l1) *CELL-SIZE-X*)
    (incf (cdr l2) (/ *CELL-SIZE-Y* 2))
    (draw-line-xy (pwin-window pwin) (car l1) (cdr l1)
		  (car l2) (cdr l2) :operation boole-xor)))


(defun draw-arc? (s1 s2)
  (help-clear-out-marks-forward `(,s1) (gensym) nil)
  (< (count-paths s1 s2 (gensym)) 2))


(defun count-paths (step target mark)
  (loop for child in (operator-succs step)
	if (equal child target) sum 1
	if (not (equal (operator-mark child) mark))
	do  (setf (operator-mark child) mark)
	and sum (count-paths child target mark)))
  


(defun help-redraw-plan (board depth)
  (declare (ignore x))
  (copy-rectangle-xy (pwin-window (aref *screen-array* depth)) 0 0
		     *window-width* *window-height* 0 0 board))


(defun redraw-plan (board &rest x)
  (declare (ignore x))
  (help-redraw-plan board *screen-pointer*))


#|   WRITE-CELL

Move ?p from ?q to ?r at (x,y).

|#

(defun write-cell (x y p? q? r? m win)
  (let* ((loc (from-cell-to-screen x y))
	 (op (window-stream-operation win)))
    (draw-rectangle-xy win (car loc) (cdr loc) *cell-size-x* *cell-size-y*)
    (setf (window-stream-operation win) 'boole-xor)
    (cond ((null q?))
	  ((equal q? 'table)
	   (write-name-onto-step p? win *col1* *row1* loc 1))
	  (t
	   (write-name-onto-step p? win *col1* *row3* loc 1)
	   (write-name-onto-step q? win *col1* *row1* loc 1)))
    (cond ((null r?))
	  ((equal r? 'table)
	   (write-name-onto-step p? win *col3* *row1* loc 1))
	  (t
	   (write-name-onto-step p? win *col3* *row3* loc 1)
	   (write-name-onto-step r? win *col3* *row1* loc 1)))
    (cond ((equal p? '+)
	   (write-name-onto-step "I" win *col2* *row2* loc 1))
	  ((equal p? '-)
	   (write-name-onto-step "F" win *col2* *row2* loc 1))
	  (t 
	   (write-name-onto-step "=" win *col2* *row2* loc 1)
	   (write-name-onto-step ">" win *col4* *row2* loc 1)))
    (if m (write-name-onto-step (format nil "~2d" m) win *col5* *row4* loc 2))
    (setf (window-stream-operation win) op)))



(defun write-name-onto-step (name win dx dy loc seq)
  (setf (window-stream-x-position win)
	(+ (car loc) (* dx *cell-size-x*))
	(window-stream-y-position win)
	(+ (cdr loc) (* dy *cell-size-y*)))
  (format win "~a" (subseq (format nil "~a" name) 0 seq)))




#|
(defun from-screen-to-cell (mouse-position)
  (let ((x (floor (- (position-x mouse-position) *margin*) *cell-size*))
	(y (floor (- (position-y mouse-position) *margin*) *cell-size*)))
    (if (not (illegal-display-position x y))
	(cons x y))))
|#


(defun from-cell-to-screen (x y)
  (cons (+ *border* (* (+ *cell-size-x* *spacing*) x) 1)
	(+ *border* (* (+ *cell-size-y* *spacing*) y) 1)))

(defun print-pcell (struct stream depth)
  (declare (ignore depth))
  (if (typep (pcell-step struct) 'operator)
      (princ (operator-class-name (operator-class (pcell-step struct))) stream)
    (princ nil stream)))



(defun mark-for-depth (op depth)
  (declare (type fixnum depth))
  (clear-out-marks `(,op) (1- depth))
  (help-mark-for-depth op depth))

(defun help-mark-for-depth (op depth)
  (declare (type fixnum depth))
  (if (> depth (operator-mark op))
      (progn
	(setf (operator-mark op) depth)
	(dolist (child (operator-succs op))
		(help-mark-for-depth child (1+ depth))))))


(defun testing nil
  (setf *ACTIVE-LEFT*
	(make-active-region :left 0
			    :bottom 0 :activate-p t
			    :width 30 :height 15
			    :parent *plan-screen*
			    :mouse-cursor *mouse-cursor-x*))
  (setf *ACTIVE-MIDDLE*
	(make-active-region :left 31
			    :bottom 0 :activate-p t
			    :width 30 :height 15
			    :parent *plan-SCREEN*
			    :mouse-cursor *mouse-cursor-+*))
  (setf *ACTIVE-RIGHT*
	(make-active-region :left 61
			    :bottom 0 :activate-p t
			    :width 30 :height 15
			    :parent *plan-SCREEN*
			    :mouse-cursor *mouse-cursor-x*))
#|  (setf (active-region-mouse-cursor-in *ACTIVE-LEFT*)
        '(mouse-cursor-left-handler))
  (setf (active-region-mouse-cursor-out *ACTIVE-LEFT*)
        '(mouse-cursor-left-handler))|#
  (setf (active-region-mouse-cursor-in *ACTIVE-middle*)
        '(mouse-cursor-middle-handler))
  (setf (active-region-mouse-cursor-out *ACTIVE-middle*)
        '(mouse-cursor-middle-handler))
  (setf (active-region-mouse-cursor-in *ACTIVE-RIGHT*)
        '(mouse-cursor-right-handler))
  (setf (active-region-mouse-cursor-out *ACTIVE-RIGHT*)
        '(mouse-cursor-right-handler))
  (modify-active-region-method *ACTIVE-LEFT* :left-button-down
                               :after 'left-button-down-handler-ar)
  (modify-active-region-method *ACTIVE-middle* :left-button-down
                               :after 'middle-button-down-handler-ar)
  (modify-active-region-method *ACTIVE-RIGHT* :left-button-down
                               :after 'right-button-down-handler-ar))

(defun mouse-cursor-left-handler (aregion mouse-state &optional event)
  "Modify appropriate variables when cursor enters an active cell"
  (declare (ignore event aregion mouse-state))
  (draw-filled-rectangle-xy *plan-screen* 0 0 30 15 :operation boole-xor))

(defun mouse-cursor-middle-handler (aregion mouse-state &optional event)
  "Modify appropriate variables when cursor enters an active cell"
  (declare (ignore event aregion mouse-state))
  (draw-filled-rectangle-xy *plan-screen* 31 0 30 15 :operation boole-xor))

(defun mouse-cursor-right-handler (aregion mouse-state &optional event)
  "Modify appropriate variables when cursor enters an active cell"
  (declare (ignore event aregion mouse-state))
  (draw-filled-rectangle-xy *plan-screen* 61 0 30 15 :operation boole-xor))

(defun left-button-down-handler-ar (aregion mouse-state &optional event)
  "Blow the blocks via this method"
  (declare (ignore event aregion mouse-state))
  (setf *next-step* t))

(defun middle-button-down-handler-ar (aregion mouse-state &optional event)
  "Blow the blocks via this method"
  (declare (ignore event aregion mouse-state))
  (setf *streamline* (not *streamline*))
  (mouse-cursor-middle-handler nil nil))

(defun right-button-down-handler-ar (aregion mouse-state &optional event)
  "Blow the blocks via this method"
  (declare (ignore event aregion mouse-state))
  (setf *graphics-on* (not *graphics-on*))
  (mouse-cursor-right-handler nil nil))
