;;;  Contains: Micellaneous common files
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)

(require :loop)

(proclaim '(optimize (speed 3) (safety 0)))


(defun choose-item-randomly (l) (if l (nth (new-random (length l)) l)))


(defun shuffle-list (l)
  (let ((len (length l)))
    (dotimes (count len l) (rotatef (nth (new-random len) l)
				    (nth (new-random len) l)))))


(defun insert-ordering (value data l)
  (let ((class (assoc value l)))
    (if class (rplacd (last class) `(,data))
      (push `(,value ,data) l)))
  l)


(defun select-ordering (l)
  (declare (special *random-goal*))
  (cond ((or (not (eql *random-goal* 'random)) (null l)))
	(t (rotatef (car l) (nth (new-random (length l)) l))))
  l)


(defun extract-ordering (l)
  (cond ((null l) nil)
	((cdar l)
	 (rplacd (car l) (select-ordering (cdar l)))
	 (pop (cdar l)))
	((car (rplaca l (cadr l)))
	 (pop (cdr l))
	 (extract-ordering l))
	(t nil)))


(defun display-bw-state (state clears?)
  (let ((stacks nil) (temp nil))
    (dolist (item state) (if (eql (car item) 'on) (push (cdr item) stacks)))
    (setf stacks (copy-tree stacks))
    (loop with l = 0
	  with nl = (length stacks) do
	  (setf l nl)
	  (setf stacks (reduce-bw-stacks stacks))
	  until (= l (setf nl (length stacks))))
    (dolist (stack stacks)
	    (if (not (eql (car (setf temp (last stack))) 'table))
		(push nil (cdr temp))))
    (loop while (even-up-bw stacks))
    (loop for i from 0 to (1- (length (car stacks))) do
	  (dolist (stack stacks (terpri))
		  (case (nth i stack)
			(table (princ "---"))
			((nil) (princ "   "))
			(t (format t " ~2a" (nth i stack))))))
    (if clears?
	(if (setf temp (remove 'clear state :test-not #'equal :key #'car))
	    (princ temp))))
  (terpri)
  nil)


(defun even-up-bw (stacks)
  (loop with flag = nil
	for ptr on stacks do
	(cond ((null (cdr ptr)) (return flag))
	      ((= (length (car ptr)) (length (cadr ptr))))
	      ((< (length (car ptr)) (length (cadr ptr)))
	       (push nil (car ptr)) (setf flag t))
	      (t (push nil (cadr ptr)) (setf flag t)))))


(defun reduce-bw-stacks (stacks)
  (let ((temp nil))
    (dolist (tuple stacks)
      (setf temp (last tuple))
      (rplacd temp (cdr (assoc (car temp) stacks)))
      (if (cdr temp) 
	  (progn (setf stacks (delete temp stacks :test #'equal))
		 (return)))))
  stacks)
