;;;  Contains: goal selection schema
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)


(require :loop)


(proclaim '(optimize (speed 3) (safety 0)))


(defun select-goal (head level)
  (case *random-goal*
    (random (select-random-goal head))
    (<      (select-alphabetical<-goal head))
    (>      (select-alphabetical>-goal head))
    (<prime (select-alphabetical<prime-goal head))
    (>prime (select-alphabetical>prime-goal head))
    (comb   (if (zerop (mod level 2))
		(select-alphabetical<-goal head)
		(select-alphabetical>-goal head)))
    (t (error "Unknown *RANDOM-GOAL* setting: ~A" *RANDOM-GOAL*))))


(defun select-alphabetical<-goal (head)
  (let ((lowest (car head)))
    (loop with sl = (format nil "~A" (pred-goal lowest))
	  with temp = ""
	  with item = (car head)
	  do (setf item (pred-aft item))
	  while (not (eq item (car head)))
	  if (or
	      (string< (setf temp (format nil "~A" (pred-goal item))) sl)
	      (and (string= temp sl)
		       (< (operator-instance (pred-op item))
			  (operator-instance (pred-op lowest)))))
	  do (setf lowest item sl temp)
	  finally (return lowest))))


(defun select-alphabetical>-goal (head)
  (let ((lowest (car head)))
    (loop with sl = (format nil "~A" (pred-goal lowest))
	  with temp = ""
	  with item = (car head)
	  do (setf item (pred-aft item))
	  while (not (eq item (car head)))
	  if (or
	      (string> (setf temp (format nil "~A" (pred-goal item))) sl)
	      (and (string= temp sl)
		       (< (operator-instance (pred-op item))
			  (operator-instance (pred-op lowest)))))
	  do (setf lowest item sl temp)
	  finally (return lowest))))


(defun select-alphabetical<prime-goal (head)
  (let ((lowest (car head)))
    (loop with sl = (format nil "~A" (pred-goal lowest))
	  with temp = ""
	  with item = (car head)
	  do (setf item (pred-aft item))
	  while (not (eq item (car head)))
	  if (or
	      (string< (setf temp (format nil "~A" (pred-goal item))) sl)
	      (and (string= temp sl)
		       (> (operator-instance (pred-op item))
			  (operator-instance (pred-op lowest)))))
	  do (setf lowest item sl temp)
	  finally (return lowest))))


(defun select-alphabetical>prime-goal (head)
  (let ((lowest (car head)))
    (loop with sl = (format nil "~A" (pred-goal lowest))
	  with temp = ""
	  with item = (car head)
	  do (setf item (pred-aft item))
	  while (not (eq item (car head)))
	  if (or
	      (string> (setf temp (format nil "~A" (pred-goal item))) sl)
	      (and (string= temp sl)
		       (> (operator-instance (pred-op item))
			  (operator-instance (pred-op lowest)))))
	  do (setf lowest item sl temp)
	  finally (return lowest))))


(defun select-random-goal (head)
  (if (car head)
      (let ((index (1+ (new-random  
			(1+ (loop with item = (car head)
				  while (not (eq (setf item (pred-aft item))
						 (car head)))
				  sum 1))))))
	(loop with item = (car head)
	      if (zerop (decf index))
	      do (return item)
	      else do (setf item (pred-aft item))))))


(defun complete-list-of-goals (head)
  (loop with item = (car head)
	collect item
	until (eq (setf item (pred-aft item)) (car head))))
