;;;  Contains: The goal ordering code for TO, the totally ordered planner
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)

(proclaim '(optimize (speed 3) (safety 0)))

(proclaim '(special *OPERATORS* *OP-CLASSES* *initial-state*
		    *final-state* *operator-number*
		    *compiled-operators* *PLAN* *random-goal*))


(defun help-get-unachieved-goals (op unach nec ach)
  (if op
      (progn
	(dolist (add (operator-class-add (operator-class op)))
	  (setf unach (modified-delete add unach ach t)))
	(dolist (del (operator-class-del (operator-class op)))
	  (setf unach (modified-delete del unach nec nil)))
	(dolist (pred (operator-pre op)) (setf (pred-? pred) 'maybe))
	(help-get-unachieved-goals
	 (car (operator-preds op)) (nconc (copy-list (operator-pre op)) unach)
	 nec ach))
    unach))


(defun get-unachieved-goals nil
  (let ((unach nil) (nec (list nil)) (ach (list nil)))
    (clear-out-marks (list *initial-operator*) 0)
    (setf unach (help-get-unachieved-goals *final-operator* nil nec ach))
    (dolist (item unach)
      (setf (pred-? item) nil)
      (circular-add nec item))
    (list nec ach)))



(defun help-adjust-unachieved-goals-pre (op unach nec ach)
  (if (and op unach)
      (progn
	(dolist (add (operator-class-add (operator-class op)))
	  (setf unach (modified-delete add unach ach t)))
	(dolist (del (operator-class-del (operator-class op)))
	  (setf unach (modified-delete del unach nec nil)))
	(help-adjust-unachieved-goals-pre
	 (car (operator-preds op)) unach nec ach))
    unach))


#|
(defun help-adjust-unachieved-goals-pre (op unach nec ach)
  (if (and op unach)
      (progn
	(dolist (add (operator-class-add (operator-class op)))
	  (setf unach (modified-delete add unach ach t)))
	(dolist (del (operator-class-del (operator-class op)))
	  (setf unach (modified-delete del unach nec nil)))
	(help-get-unachieved-goals (car (operator-preds op)) unach nec ach))
    unach))
|#


(defun help-adjust-unachieved-goals-add (add new-op nec ach trash)
  (if (and new-op (car nec))
      (let ((continue t) (class (operator-class new-op)))
	(dolist (item (operator-pre new-op))
	  (cond ((not (equal (pred-goal item) add)))
		((pred-? item)
		 (setf continue nil)
		 (return))
		(t 
		 (circular-add ach item)
		 (setf (pred-? item) t)
		 (push item trash)
		 (return))))
	(if (and continue
		 (not (member add (operator-class-add class) :test #'equal))
		 (not (member add (operator-class-del class) :test #'equal)))
	    (help-adjust-unachieved-goals-add
	     add (car (operator-succs new-op)) nec ach trash)
	  trash))
    trash))


(defun help-adjust-unachieved-goals-del (del new-op nec ach trash op)
  (if (and new-op (car ach))
      (let ((continue t) (class (operator-class new-op)))
	(dolist (item (operator-pre new-op))
	  (cond ((not (equal (pred-goal item) del)))
		((not (pred-? item))
		 (setf continue nil)
		 (return))
		(t
		 (circular-add nec item)
		 (setf (pred-? item) nil)
		 (push item trash)
		 (return))))
	(if (and continue
		 (not (member del (operator-class-add class) :test #'equal))
		 (not (member del (operator-class-del class) :test #'equal)))
	    (help-adjust-unachieved-goals-del
	     del (car (operator-succs new-op)) nec ach trash op)
	  trash))
    trash))


#| ADJUST-UNACHIEVED-GOALS

This assumes that the operator is already placed within a Totally
ordered plan.

|#

(defun adjust-unachieved-goals (op complete-cl)
  (let ((nec (first complete-cl)) (ach (second complete-cl))
	(new-adds nil) (new-dels nil))
    (dolist (pred (operator-pre op)) (setf (pred-? pred) 'maybe))
    (dolist (item (help-adjust-unachieved-goals-pre
		   (car (operator-preds op))
		   (copy-list (operator-pre op)) nec ach))
      (circular-add nec item)
      (setf (pred-? item) nil))
    (dolist (add (operator-class-add (operator-class op)))
      (setf new-adds (help-adjust-unachieved-goals-add
		      add (car (operator-succs op)) nec ach new-adds)))
    (dolist (del (operator-class-del (operator-class op)))
      (setf new-dels (help-adjust-unachieved-goals-del
		      del (car (operator-succs op)) nec ach new-dels op)))
    (values new-adds new-dels)))


(defun readjust-unach-goals (cl new-adds new-dels)
  (declare (optimize (speed 3) (safety 0)))
  (dolist (item new-adds)
	  (declare (type pred item))
	  (circular-add (first cl) item)
	  (setf (pred-? item) nil))
  (dolist (item new-dels)
	  (declare (type pred item))
	  (circular-add (second cl) item)
	  (setf (pred-? item) t)))
