;;;  Contains: Search algorithms with manhatten distance heuristic for TO
;;;  (for TileWorld domain)
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)
;;;  Created: Fri Dec 20 15:15:03 1991



(defvar *tw-goal-hash* nil)
(defvar *tw-init-hash* nil)
(defvar *tw-type-hash* nil)

(defun man-dist (op)
  (let ((h 0))
    (loop
     with del = nil
     with add = nil
     with object = nil
     for bindings in *tw-init-hash*
     do (if (not (setf object (assoc '?a bindings)))
	    (error "?A not bound in ~A" bindings))
     (setf del (car (get-all-binding-lists
		     '((at ?a ?x ?y))
		     (operator-class-del (operator-class op)) `(,object))))
     (setf add (car (get-all-binding-lists
		     '((at ?a ?x ?y))
		     (operator-class-add (operator-class op)) `(,object))))
     (if (and add del)
	 (progn
	   (incf h (+ (abs (- (cdr (assoc '?x bindings)) (cdr (assoc '?x del))))
		      (abs (- (cdr (assoc '?y bindings)) (cdr (assoc '?y del))))))
	   (decf h (+ (abs (- (cdr (assoc '?x bindings)) (cdr (assoc '?x add))))
		      (abs (- (cdr (assoc '?y bindings)) (cdr (assoc '?y add)))))))))
    h))


(defun tw-mod-to-extend-fast (complete-cl &optional (level 0))
  (cond
   ((null (caar complete-cl)) t)
   ((not (< level *depth-level*)) nil)
   (t (let ((goal-to-achieve (select-goal (first complete-cl) level)) (op nil)
	    (operators nil) (heuristic nil))
	(if *debug* (format *debug* "G: ~A~%" (pred-goal goal-to-achieve)))
	(setf operators (achievement-operators (pred-goal goal-to-achieve)))
	(if (eql *random-goal* 'random) (shuffle-list operators))
	(dolist (op-class operators nil)
	  (setf op (create-operator-instance op-class))
	  (if *debug* (format *debug* "OP: ~A~%"
			      (operator-class-name (operator-class op))))
	  (loop with position = (pred-op goal-to-achieve)
		while (setf position (car (operator-preds position))) do
		(place-operator-after op position)
		(setf heuristic (insert-ordering
				 (man-dist op) `(,op-class ,position) heuristic))
		(remove-operator-from-plan op)
		until (member (pred-goal goal-to-achieve)
			      (operator-class-del (operator-class position))
			      :test #'equal))
	  (destroy-operator-instance))
	(setf heuristic (sort heuristic #'< :key #'car))
	(loop with tuple = nil
	      with position = nil
	      while (setf tuple (extract-ordering heuristic)) do
	      (setf op (create-operator-instance (car tuple))
		    position (nth 1 tuple))
	      (if *debug* (format *debug* "OP: ~A~%"
				  (operator-class-name (operator-class op))))
	      (place-operator-after op position)
	      (graphics-add-step *plan* level)
	      (multiple-value-bind
	       (n-adds n-dels) (adjust-unachieved-goals op complete-cl)
	       (if *debug* (format *debug* "~A ~A~%" *plan* position))
	       (incf *visits*)
	       (if (tw-mod-to-extend-fast complete-cl (1+ level)) (return t))
	       (remove-operator-from-plan op)
	       (readjust-unach-goals complete-cl n-adds n-dels)
	       (graphics-del-step *plan* level))
	      (destroy-operator-instance))))))


(defun tw-mod-ua-extend-fast (complete-cl &optional (level 0))
  (cond
   ((null (caar complete-cl)) t)
   ((not (< level *depth-level*)) nil)
   (t (let ((pre (select-goal (first complete-cl) level)) (op nil)
	    (deleter nil) (heuristic nil))
	(if *debug* (format *debug* "G: ~A~%" (pred-goal pre)))
	(dolist (op-class (achievement-operators (pred-goal pre)))
	  (setf op (create-operator-instance op-class)
		deleter (find-deleter pre))
	  (add-ordering deleter op)
	  (add-ordering op (pred-op pre))
	  (loop with state = (disambiguate-setup op #'always-after)
		with flag  = (if (nth 1 state) t)
		with stack = nil
		with stuff = nil
		while (setf state (disambiguate-forward op state)) do
		(setf stack (if flag (copy-stack-from-state state) nil)
		      stuff (list op-class stack deleter))
		(setf heuristic (insert-ordering (man-dist op) stuff heuristic))
		(if flag (setf state (disambiguate-backtrack op state)))
		while (and state flag))
	  (remove-ordering op (pred-op pre))
	  (remove-ordering deleter op)
	  (destroy-operator-instance))
	(setf heuristic (sort heuristic #'< :key #'car))
	(loop with tuple = nil
	      with stack = nil
	      while (setf tuple (extract-ordering heuristic)) do
	      (setf op (create-operator-instance (nth 0 tuple))
		    stack (nth 1 tuple)
		    deleter (nth 2 tuple))
	      (add-ordering deleter op)
	      (add-ordering op (pred-op pre))
	      (if *debug* (format *debug* "OP: ~A~%"
				  (operator-class-name (operator-class op))))
	      (reinstate op stack)
	      (graphics-add-step *plan* level)
	      (multiple-value-bind
	       (n-adds n-dels) (ua-adjust-unachieved-goals op complete-cl)
	       (if *debug* (progn (format t "U: ~D, A: ~D, " (length n-dels)
					  (length n-adds)) (ua-order-plan)))
	       (incf *visits*)
	       (if (tw-mod-ua-extend-fast complete-cl (1+ level)) (return t))
	       (deinstate op stack)
	       (readjust-unach-goals complete-cl n-adds n-dels)
	       (graphics-del-step *plan* level))
	      (remove-ordering op (pred-op pre))
	      (remove-ordering deleter op)
	      (destroy-operator-instance))))))


(defun to-tw nil
  (to)
  (setf *tw-goal-hash* (get-all-binding-lists '((at ?a ?x ?y)) *final-state* nil))
  (setf *tw-type-hash* (mapcar #'(lambda (x) (list (assoc '?a x))) *tw-goal-hash*))
  (setf *tw-init-hash*
	(apply #'nconc (mapcar #'(lambda (x) (get-all-binding-lists
					      '((at ?a ?x ?y)) *initial-state* x))
			       *tw-type-hash*)))
  (let ((t1 (get-internal-run-time))
	(result (tw-mod-to-extend-fast (get-unachieved-goals)))
	(t2 (get-internal-run-time)))
    (format *out-stream* "Number of calls: ~A~%" *visits*)
    (dolist (i *plan*) (format *out-stream* "~A~%" i))
    `(,result ,*visits* ,(/ (- t2 t1) internal-time-units-per-second 1.0))))


(defun ua-tw nil
  (ua)
  (setf *tw-goal-hash* (get-all-binding-lists '((at ?a ?x ?y)) *final-state* nil))
  (setf *tw-type-hash* (mapcar #'(lambda (x) (list (assoc '?a x))) *tw-goal-hash*))
  (setf *tw-init-hash*
	(apply #'nconc (mapcar #'(lambda (x) (get-all-binding-lists
					      '((at ?a ?x ?y)) *initial-state* x))
			       *tw-type-hash*)))
  (let ((t1 (get-internal-run-time))
	(result (tw-mod-ua-extend-fast (ua-get-unachieved-goals)))
	(t2 (get-internal-run-time)))
    (format *out-stream* "Number of calls: ~A~%" *visits*)
    (dolist (i *plan*) (format *out-stream* "~A~%" i))
    `(,result ,*visits* ,(/ (- t2 t1) internal-time-units-per-second 1.0))))

