;;;  Contains: Blocksworld problems used in experiments.
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)

(defvar *operator-branching* 5)  ;; All problems should set this variable

#| *OPERATOR-BRANCHING*

   For full-set   is (N-1)^2
   For restricted is (N-1)

It represents the maximum branching factor for the operator selection
(not placement which is based on depth as well.

|#


(setf *random-goal* 'random)
(setf *node-limit* 100000)

(defun problem (num)
  (case
   num
   ( 1 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (ON A TABLE)
	       (CLEAR A) (ON C E) (CLEAR C) (ON E B) (ON B D) (ON D TABLE))
	     *final-state*
	     '((ON A TABLE) (ON E D))
	     *depth-level* 4 *operator-branching* 5))
   ( 2 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (ON E B)
	       (CLEAR E) (ON B C) (ON C TABLE) (ON A D) (CLEAR A) (ON D TABLE))
	     *final-state*
	     '((ON D TABLE) (ON C D))
	     *depth-level* 4 *operator-branching* 5))
   ( 3 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (ON C TABLE)
	       (CLEAR C) (ON D A) (CLEAR D) (ON A TABLE) (ON F E) (CLEAR F)
	       (ON E TABLE))
	     *final-state*
	     '((ON C F) (ON E A))
	     *depth-level* 4 *operator-branching* 5))
   ( 4 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK E) (BLOCK F) (ON C B)
	       (CLEAR C) (ON B F) (ON F A) (ON A E) (ON E TABLE))
	     *final-state*
	     '((ON E TABLE) (ON F TABLE) (ON A C))
	     *depth-level* 5 *operator-branching* 5))
   ( 5 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK E) (BLOCK F) (ON F TABLE)
	       (CLEAR F) (ON A E) (CLEAR A) (ON E TABLE) (ON B C) (CLEAR B)
	       (ON C TABLE))
	     *final-state*
	     '((ON A C) (ON B F) (CLEAR A))
	     *depth-level* 4 *operator-branching* 5))
   ( 6 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F)
	       (ON C D) (CLEAR C) (ON D TABLE) (ON E B) (CLEAR E) (ON B TABLE)
	       (ON A F) (CLEAR A)
	       (ON F TABLE))
	     *final-state*
	     '((ON A B) (CLEAR A) (ON B D))
	     *depth-level* 5 *operator-branching* 6))
   ( 7 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (ON D B) (CLEAR D) (ON B TABLE) (ON E G) (CLEAR E) (ON G TABLE)
	       (ON F A) (CLEAR F)
	       (ON A TABLE))
	     *final-state*
	     '((ON D A) (ON E F))
	     *depth-level* 5 *operator-branching* 6))
   ( 8 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK G) (ON A E)
	       (CLEAR A) (ON E G) (ON G D) (ON D TABLE) (ON C TABLE) (CLEAR C))
	     *final-state*
	     '((ON D C) (CLEAR G))
	     *depth-level* 4 *operator-branching* 5))
   (9 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK G)
	       (ON C TABLE) (CLEAR C) (ON G E) (CLEAR G) (ON E TABLE) (ON A D)
	       (CLEAR A) (ON D B)
	       (ON B TABLE))
	     *final-state*
	     '((ON G C) (CLEAR B) (ON A D))
	     *depth-level* 5 *operator-branching* 6))
   (10 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (ON E C) (CLEAR E) (ON C TABLE) (ON B F) (CLEAR B) (ON F TABLE)
	       (ON G D) (CLEAR G)
	       (ON D TABLE))
	     *final-state*
	     '((CLEAR C) (ON G TABLE) (ON D F))
	     *depth-level* 4 *operator-branching* 6))
   (11 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (ON A F) (CLEAR A) (ON F TABLE) (ON G E) (CLEAR G) (ON E D)
	       (ON D C) (ON C TABLE))
	     *final-state*
	     '((ON D TABLE) (CLEAR A) (ON F D))
	     *depth-level* 5 *operator-branching* 6))
   (12 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK G)
	       (ON D TABLE) (CLEAR D) (ON A B) (CLEAR A) (ON B C) (ON C TABLE)
	       (ON G E) (CLEAR G)
	       (ON E TABLE))
	     *final-state*
	     '((CLEAR E) (ON E TABLE) (ON C D))
	     *depth-level* 4 *operator-branching* 6))
   (13 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK E) (BLOCK G) (ON G E) (CLEAR G)
	       (ON E A) (ON A B) (ON B TABLE))
	     *final-state*
	     '((ON B G) (ON E B) (ON A TABLE))
	     *depth-level* 5 *operator-branching* 4))
   (14 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK F) (BLOCK H)
	       (ON A F) (CLEAR A) (ON F TABLE) (ON D C) (CLEAR D) (ON C TABLE)
	       (ON H B) (CLEAR H)
	       (ON B TABLE))
	     *final-state*
	     '((CLEAR F) (ON D B))
	     *depth-level* 4 *operator-branching* 6))
   (15 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK B) (BLOCK C) (BLOCK E) (BLOCK F) (BLOCK G) (BLOCK H)
	       (ON E TABLE) (CLEAR E) (ON G C) (CLEAR G) (ON C TABLE) (ON H B)
	       (CLEAR H) (ON B F) (ON F TABLE))
	     *final-state*
	     '((CLEAR F) (ON E C))
	     *depth-level* 4 *operator-branching* 6))
   (16 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK E) (BLOCK F) (BLOCK G) (BLOCK H)
	       (ON F TABLE) (CLEAR F) (ON A C) (CLEAR A) (ON C TABLE) (ON H E)
	       (CLEAR H) (ON E G) (ON G TABLE))
	     *final-state*
	     '((ON E A) (ON A TABLE) (ON F H))
	     *depth-level* 5 *operator-branching* 6))
   (17 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK H)
	       (ON H TABLE) (CLEAR H) (ON E D) (CLEAR E) (ON D B) (ON B C)
	       (ON C TABLE) (ON F TABLE) (CLEAR F))
	     *final-state*
	     '((CLEAR H) (ON E B) (ON C F))
	     *depth-level* 5 *operator-branching* 6))
   (18 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK E) (BLOCK F) (BLOCK G) (BLOCK H)
	       (ON G H) (CLEAR G) (ON H E) (ON E TABLE) (ON F B) (CLEAR F)
	       (ON B A) (ON A TABLE))
	     *final-state*
	     '((ON E A) (ON A G) (CLEAR F))
	     *depth-level* 6 *operator-branching* 6))
   (19 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F)
	       (BLOCK G) (BLOCK H) (ON F TABLE) (CLEAR F) (ON H TABLE)
	       (CLEAR H) (ON G D) (CLEAR G) (ON D A) (ON A TABLE) (ON E B)
	       (CLEAR E) (ON B TABLE) (ON C TABLE) (CLEAR C))
	     *final-state*
	     '((ON E TABLE) (CLEAR H) (ON F C) (ON A E))
	     *depth-level* 5 *operator-branching* 8))
   (20 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (BLOCK H) (ON G F) (CLEAR G) (ON F TABLE) (ON A E) (CLEAR A)
	       (ON E C) (ON C TABLE) (ON H D) (CLEAR H) (ON D TABLE))
	     *final-state*
	     '((ON H C) (CLEAR E) (ON G TABLE) (ON D TABLE))
	     *depth-level* 5 *operator-branching* 7))
   (21 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK B) (BLOCK C) (BLOCK D) (BLOCK F) (BLOCK G) (BLOCK H) 
	       (ON H F) (CLEAR H) (ON F C) (ON C D) (ON D TABLE) (ON B G)
	       (CLEAR B) (ON G TABLE))
	     *final-state*
	     '((ON H D) (ON C TABLE) (CLEAR B) (ON B H))
	     *depth-level* 6 *operator-branching* 6))
   (22 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK G)
	       (BLOCK H) (ON G TABLE) (CLEAR G) (ON E A) (CLEAR E)
	       (ON A TABLE) (ON H B) (CLEAR H) (ON B TABLE) (ON C D) (CLEAR C)
	       (ON D TABLE))
	     *final-state*
	     '((ON B A) (ON C TABLE) (ON E TABLE) (ON G TABLE))
	     *depth-level* 4 *operator-branching* 7))
   (23 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK G)
	       (ON C D) (CLEAR C) (ON D G) (ON G B) (ON B A) (ON A TABLE)
	       (ON E TABLE) (CLEAR E))
	     *final-state*
	     '((ON D TABLE) (ON B A) (ON G E) (CLEAR D))
	     *depth-level* 4 *operator-branching* 6))
   (24 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F)
	       (BLOCK G) (BLOCK H) (ON G C) (CLEAR G) (ON C E) (ON E TABLE)
	       (ON B TABLE) (CLEAR B) (ON A D) (CLEAR A) (ON D H) (ON H TABLE)
	       (ON F TABLE) (CLEAR F))
	     *final-state*
	     '((CLEAR G) (ON B H) (ON F TABLE) (ON C A))
	     *depth-level* 6 *operator-branching* 8))
  (25 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (BLOCK H) (ON F TABLE) (CLEAR F) (ON D H) (CLEAR D) (ON H C)
	       (ON C TABLE) (ON A G) (CLEAR A) (ON G E) (ON E TABLE))
	     *final-state*
	     '((ON G TABLE) (ON C A) (CLEAR G) (CLEAR F))
	     *depth-level* 5 *operator-branching* 7))
   (26 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F)
	       (ON D B) (CLEAR D) (ON B TABLE) (ON C A) (CLEAR C) (ON A E)
	       (ON E F) (ON F TABLE))
	     *final-state*
	     '((ON C TABLE) (ON E C) (ON B A))
	     *depth-level* 6 *operator-branching* 6))
   (27 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK F) (BLOCK G)
	       (BLOCK H) (ON F A) (CLEAR F) (ON A TABLE) (ON C H) (CLEAR C)
	       (ON H TABLE) (ON G B)
	       (CLEAR G) (ON B D) (ON D TABLE))
	     *final-state*
	     '((ON B TABLE) (ON D A) (ON C B))
	     *depth-level* 6 *operator-branching* 7))
   (28 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK E) (BLOCK F) (BLOCK H)
	       (CLEAR F) (ON F E) (ON E B) (ON B A) (ON A TABLE) (ON C H)
	       (CLEAR C) (ON H TABLE))
	     *final-state*
	     '((CLEAR C) (ON H A) (CLEAR B) (ON F H))
	     *depth-level* 6 *operator-branching* 6))
   (29 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (ON B G) (CLEAR B) (ON G C) (ON C D) (ON D TABLE) (ON F E)
	       (CLEAR F) (ON E TABLE))
	     *final-state*
	     '((CLEAR D) (CLEAR C) (ON C F) (ON E G))
	     *depth-level* 6 *operator-branching* 6))
   (30 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F)
	       (BLOCK G) (BLOCK H) (ON G D) (CLEAR G) (ON D TABLE) (ON E A)
	       (CLEAR E) (ON A H) (ON H C) (ON C TABLE) (ON B F) (CLEAR B)
	       (ON F TABLE))
	     *final-state*
	     '((ON A C) (ON B F) (ON F TABLE) (ON D A))
	     *depth-level* 6 *operator-branching* 8))
   (31 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (BLOCK G)
	       (BLOCK H) (BLOCK I) (ON H D) (CLEAR H) (ON D TABLE) (ON E I)
	       (CLEAR E) (ON I G) (ON G TABLE) (ON F C) (CLEAR F) (ON C TABLE)
	       (ON A TABLE) (CLEAR A))
	     *final-state*
	     '((ON F A) (ON I TABLE) (CLEAR H) (ON H F))
	     *depth-level* 6 *operator-branching* 8))
   (32 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK G)
	       (BLOCK H) (BLOCK I) (CLEAR E) (ON E H) (ON H TABLE) (ON G A)
	       (CLEAR G) (ON A TABLE) (ON D I) (CLEAR D) (ON I TABLE)
	       (ON C B) (CLEAR C) (ON B TABLE))
	     *final-state*
	     '((CLEAR G) (ON B I) (ON C A) (CLEAR H))
	     *depth-level* 6 *operator-branching* 8))
   (33 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK C) (BLOCK D) (BLOCK E) (ON A E) (CLEAR A)
	       (ON E TABLE) (ON C D) (CLEAR C) (ON D TABLE))
	     *final-state*
	     '((ON C E) (ON E TABLE))
	     *depth-level* 3 *operator-branching* 4))
   (34 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK B) (BLOCK D) (BLOCK E) (BLOCK F) (ON F E) (CLEAR F)
	       (ON E D) (ON D B) (ON B TABLE))
	     *final-state*
	     '((ON D TABLE) (CLEAR D))
	     *depth-level* 3 *operator-branching* 4))
   (35 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK D) (BLOCK E) (ON A TABLE) (CLEAR A)
	       (ON D B) (CLEAR D) (ON B E) (ON E TABLE))
	     *final-state*
	     '((ON E A) (ON D TABLE))
	     *depth-level* 3 *operator-branching* 4))
   (36 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F)
	       (ON C D) (CLEAR C) (ON D TABLE) (ON E TABLE) (CLEAR E)
	       (ON F TABLE) (CLEAR F) (ON B A) (CLEAR B) (ON A TABLE))
	     *final-state*
	     '((ON F C) (ON B A) (ON E D))
	     *depth-level* 3 *operator-branching* 5))
   (37 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK D) (BLOCK E) (BLOCK F) (ON F TABLE) (CLEAR F)
	       (ON A TABLE) (CLEAR A) (ON D E) (CLEAR D) (ON E TABLE))
	     *final-state*
	     '((ON F D) (CLEAR F) (ON D A))
	     *depth-level* 3 *operator-branching* 4))
   (38 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (ON C TABLE) (CLEAR C) (ON A B)
	       (CLEAR A) (ON B TABLE))
	     *final-state*
	     '((ON C A) (ON B C))
	     *depth-level* 3 *operator-branching* 3))
   (39 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK D) (BLOCK E) (ON B D) (CLEAR B) 
	       (ON D E) (ON E TABLE) (ON A TABLE) (CLEAR A))
	     *final-state*
	     '((ON D B) (CLEAR A))
	     *depth-level* 3 *operator-branching* 4))
   (40 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK F) (ON F A) (CLEAR F)
	       (ON A TABLE) (ON C B) (CLEAR C) (ON B TABLE))
	     *final-state*
	     '((CLEAR C) (ON F B))
	     *depth-level* 3 *operator-branching* 4))
   (41 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK D) (BLOCK E) (BLOCK F) (ON B D)
	       (CLEAR B) (ON D TABLE) (ON A E) (CLEAR A) (ON E TABLE)
	       (ON F TABLE) (CLEAR F))
	     *final-state*
	     '((ON D F) (CLEAR E))
	     *depth-level* 3 *operator-branching* 5))
   (42 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK E) (BLOCK F) (ON C B) 
	       (CLEAR C) (ON B A) (ON A TABLE) (ON E F) (CLEAR E) (ON F TABLE))
	     *final-state*
	     '((CLEAR A) (ON C TABLE) (ON E TABLE))
	     *depth-level* 3 *operator-branching* 5))
   (43 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK C) (BLOCK D) (BLOCK E) (BLOCK F) (ON D C) (CLEAR D)
	       (ON C E) (ON E TABLE) (ON F TABLE) (CLEAR F))
	     *final-state*
	     '((ON C F) (CLEAR C) (ON F TABLE))
	     *depth-level* 3 *operator-branching* 4))
   (44 (load-restricted-ops)
       (setf *initial-state*
	     '((BLOCK A) (BLOCK B) (BLOCK C) (BLOCK E) (BLOCK F) (BLOCK G)
	       (BLOCK H) (ON H TABLE) (CLEAR H) (ON E TABLE) (CLEAR E)
	       (ON B A) (CLEAR B) (ON A C) (ON C G) (ON G TABLE) (ON F TABLE)
	       (CLEAR F))
	     *final-state*
	     '((ON F E) (ON E A) (ON C H))
	     *depth-level* 6 *operator-branching* 7)))
  (format t "Initial State:~%")
  (display-bw-state *initial-state* nil)
  (format t "Final State:~%")
  (display-bw-state *final-state* t)
  (format t "Depth Level: ~A~%" *depth-level*))


