;;;  Contains: Strips World operators.
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)
;;;  Created: Sun Jan 12 16:48:05 1992

(setf *operators*
      '(((move-between ?robot ?room1 ?door ?room2)
	 (static ((robot ?robot) (connects ?room1 ?door ?room2)))
	 (pre ((in ?robot ?room1) (next-to ?robot ?door)))
	 (add ((in ?robot ?room2)))
	 (del ((in ?robot ?room1))))
	((move-to-door ?robot ?door ?room ?door-old)
	 (static ((robot ?robot)
		  (connects ?room ?door ?room-another)
		  (connects ?room ?door-old ?room-third)
		  (not-equal ?door ?door-old)))
	 (pre ((in ?robot ?room) (next-to ?robot ?door-old)))
	 (add ((next-to ?robot ?door)))
	 (del ((next-to ?robot ?door-old))))
	((pick-up ?robot ?object ?room)
	 (static ((room ?room) (object ?object) (robot ?robot)))
	 (pre ((in ?robot ?room) (in ?object ?room)))
	 (add ((holding ?robot ?object)))
	 (del ((in ?object ?room))))
	((put-down ?robot ?object ?room)
	 (static ((room ?room) (object ?object) (robot ?robot)))
	 (pre ((in ?robot ?room) (holding ?robot ?object)))
	 (add ((in ?object ?room)))
	 (del ((holding ?robot ?object))))
	))



#| CREATE-INITIAL-SW-STATE

Calling this function like so:

(create-initial-sw-state
 '((1 1 5)	;; Place DOOR 1 between ROOM 1 and ROOM 5
   (2 2 5)	;; Place DOOR 2 between ROOM 2 and ROOM 5
   (3 3 5)	;; Place DOOR 3 between ROOM 3 and ROOM 5
   (4 4 5))	;; Place DOOR 4 between ROOM 4 and ROOM 5
 '((1 1 1))	;; Place ROBOT 1 in ROOM 1 next to DOOR 1
 '((1 1)	;; Place OBJECT 1 in ROOM 1
   (2 1)	;; Place OBJECT 2 in ROOM 1
   (3 1)))	;; Place OBJECT 3 in ROOM 1

will produce the following:

  '((ROOM ROOM4) (ROOM ROOM3) (ROOM ROOM2) (ROOM ROOM5) (ROOM ROOM1)
    (ROBOT ROBOT1)
    (OBJECT OBJECT3) (OBJECT OBJECT2) (OBJECT OBJECT1) 
    (DOOR DOOR4) (DOOR DOOR3) (DOOR DOOR2) (DOOR DOOR1) 
    (CONNECTS ROOM5 DOOR1 ROOM1) (CONNECTS ROOM5 DOOR2 ROOM2)
    (CONNECTS ROOM5 DOOR3 ROOM3) (CONNECTS ROOM5 DOOR4 ROOM4)
    (CONNECTS ROOM1 DOOR1 ROOM5) (CONNECTS ROOM2 DOOR2 ROOM5)
    (CONNECTS ROOM3 DOOR3 ROOM5) (CONNECTS ROOM4 DOOR4 ROOM5)
    (IN ROBOT1 ROOM1)
    (IN OBJECT1 ROOM1) (IN OBJECT2 ROOM1) (IN OBJECT3 ROOM1)
    (NEXT-TO ROBOT1 DOOR1))






(setf *initial-state*
      (create-initial-sw-state
       '((1 1 5) (2 2 5) (3 3 5) (4 4 5)) '((1 1 1))
       '((1 1) (2 1) (3 1)))
      *final-state* '((in robot1 room2) (in object1 room2))
      *depth-level* 5 *operator-branching* 5)

|#

(defun make-sym (a b) (intern (string-upcase (format nil "~A~A" a b))))


(defun create-initial-sw-state 
  (room-stats robot-stats object-stats)
  (let ((s0 (mapcar #'(lambda (x) (list 'connects (make-sym 'room (caddr x))
					(make-sym 'door (cadr x)) 
					(make-sym 'room (car x))))
		    room-stats))
	(s1 (mapcar #'(lambda (x) (list 'connects (make-sym 'room (car x))
					(make-sym 'door (cadr x)) 
					(make-sym 'room (caddr x))))
		    room-stats))
	(s2 (mapcar #'(lambda (x) (list 'in (make-sym 'robot (car x))
					(make-sym 'room (cadr x))))
		    robot-stats))
	(s3 (mapcar #'(lambda (x) (list 'in (make-sym 'object (car x))
					(make-sym 'room (cadr x))))
		    object-stats))
	(s4 (mapcar #'(lambda (x) (list 'next-to (make-sym 'robot (car x))
					(make-sym 'door (caddr x))))
		    robot-stats))
	(rooms nil) (robots nil) (objects nil) (doors nil))
    (dolist (s s1) (pushnew (nth 1 s) rooms) (pushnew (nth 3 s) rooms))
    (dolist (s s2) (pushnew (nth 2 s) rooms))
    (dolist (s s3) (pushnew (nth 2 s) rooms))
    (setf rooms (mapcar #'(lambda (x) (list 'room x)) rooms))
    (dolist (s s2) (pushnew (nth 1 s) robots))
    (dolist (s s4) (pushnew (nth 1 s) robots))
    (setf robots (mapcar #'(lambda (x) (list 'robot x)) robots))
    (dolist (s s3) (pushnew (nth 1 s) objects))
    (setf objects (mapcar #'(lambda (x) (list 'object x)) objects))
    (dolist (s s1) (pushnew (nth 2 s) doors))
    (dolist (s s4) (pushnew (nth 2 s) doors))
    (setf doors (mapcar #'(lambda (x) (list 'door x)) doors))
    (nconc rooms robots objects doors s0 s1 s2 s3 s4)))
