;;;  Contains: TileWorld problems
;;;  Author:  Andy Philips (abp@goedel.arc.nasa.gov)
;;;  Created: Thu Dec 19 16:51:52 1991


(defvar *operator-branching* 5)  ;; All problems should set this variable

(setf *operators*
      '(((move-east ?a ?x0 ?y)
	 (static
	  ((agent ?a) (incf ?x0 ?x1)))
	 (pre
	  ((at ?a ?x0 ?y) (empty ?x1 ?y)))
	 (add
	  ((at ?a ?x1 ?y) (empty ?x0 ?y)))
	 (del
	  ((at ?a ?x0 ?y) (empty ?x1 ?y))))
	((move-west ?a ?x1 ?y)
	 (static
	  ((agent ?a) (incf ?x0 ?x1)))
	 (pre
	  ((at ?a ?x1 ?y) (empty ?x0 ?y)))
	 (add
	  ((at ?a ?x0 ?y) (empty ?x1 ?y)))
	 (del
	  ((at ?a ?x1 ?y) (empty ?x0 ?y))))
	((move-north ?a ?x ?y0)
	 (static
	  ((agent ?a) (incf ?y0 ?y1)))
	 (pre
	  ((at ?a ?x ?y0) (empty ?x ?y1)))
	 (add
	  ((at ?a ?x ?y1) (empty ?x ?y0)))
	 (del
	  ((at ?a ?x ?y0) (empty ?x ?y1))))
	((move-south ?a ?x ?y1)
	 (static
	  ((agent ?a) (incf ?y0 ?y1)))
	 (pre
	  ((at ?a ?x ?y1) (empty ?x ?y0)))
	 (add
	  ((at ?a ?x ?y0) (empty ?x ?y1)))
	 (del
	  ((at ?a ?x ?y1) (empty ?x ?y0))))))


#| CREATE-INITIAL-TW-STATE

Calling this function like so:

  (create-initial-tw-state '((a1 0 0)) '((b1 1 1) (b2 1 2)) 3 3)

will produce the following:

  '((agent a1) (at a1 0 0) (at b1 1 1) (at b2 1 2) (incf 0 1) (incf 1 2)
    (empty 0 1) (empty 0 2) (empty 1 0) (empty 2 0) (empty 2 1) (empty 2 2))

|#

(defun create-initial-tw-state (agent-locs block-locs x y)
  (let ((st (nconc (mapcar #'(lambda (x) (list 'agent (car x))) agent-locs)
		   (mapcar #'(lambda (x) (push 'at x)) agent-locs)
		   (mapcar #'(lambda (x) (push 'at x)) block-locs)
		   (loop for i from 1 to (1- (max x y))
			 collect (list 'incf (1- i) i))))
	(map (loop for dx from 0 to (1- x)
		   nconc (loop for dy from 0 to (1- y)
			       collect (list 'empty dx dy)))))
    (dolist (al agent-locs)
	    (setf map (delete (cdr al) map :test #'equal :key #'cdr)))
    (dolist (al block-locs)
	    (setf map (delete (cdr al) map :test #'equal :key #'cdr)))
    (nconc st map)))
